from django.conf import settings

DRF_JWT_AUTH = getattr(settings, 'DRF_JWT_AUTH', {})

ALGORITHM = DRF_JWT_AUTH.get('ALGORITHM', 'HS256')
SIGNING_KEY = DRF_JWT_AUTH.get('SIGNING_KEY', settings.SECRET_KEY)
AUTH_HEADER = DRF_JWT_AUTH.get('AUTH_HEADER', 'Bearer')
QUERY_PARAM_NAME = DRF_JWT_AUTH.get('QUERY_PARAM_NAME', 'jwt')
EXPIRATION_DATE_FIELD = DRF_JWT_AUTH.get('EXPIRATION_DATE_FIELD', 'exp')
